@@ -184,10 +184,15 @@ class S3FileSystem(AsyncFileSystem):
         Whether to use anonymous connection (public buckets only). If False,
         uses the key/secret given, or boto's credential resolver (client_kwargs,
         environment, variables, config files, EC2 IAM server, in that order)
+    endpoint_url : string (None)
+        Use this endpoint_url, if specified. Needed for connecting to non-AWS
+        S3 buckets. Takes precedence over `endpoint_url` in client_kwargs.
     key : string (None)
-        If not anonymous, use this access key ID, if specified
+        If not anonymous, use this access key ID, if specified. Takes precedence
+        over `aws_access_key_id` in client_kwargs.
     secret : string (None)
-        If not anonymous, use this secret access key, if specified
+        If not anonymous, use this secret access key, if specified. Takes
+        precedence over `aws_secret_access_key` in client_kwargs.
     token : string (None)
         If not anonymous, use this security token, if specified
     use_ssl : bool (True)
@@ -257,6 +262,7 @@ class S3FileSystem(AsyncFileSystem):
     def __init__(
         self,
         anon=False,
+        endpoint_url=None,
         key=None,
         secret=None,
         token=None,
@@ -286,6 +292,8 @@ class S3FileSystem(AsyncFileSystem):
         if password:
             secret = password
 
+        self.endpoint_url = endpoint_url
+
         self.anon = anon
         self.key = key
         self.secret = secret
@@ -460,6 +468,7 @@ class S3FileSystem(AsyncFileSystem):
             aws_access_key_id=self.key,
             aws_secret_access_key=self.secret,
             aws_session_token=self.token,
+            endpoint_url=self.endpoint_url,
         )
         init_kwargs = {
             key: value
@@ -1501,9 +1510,7 @@ class S3FileSystem(AsyncFileSystem):
         >>> mys3file.setxattr(copy_kwargs={'ContentType': 'application/pdf'},
         ...     attribute_1='value1')  # doctest: +SKIP
 
-
-        .. Metadata Reference:
-        http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-metadata
+        .. _Metadata Reference: http://docs.aws.amazon.com/AmazonS3/latest/dev/UsingMetadata.html#object-metadata
         """
 
         kw_args = {k.replace("_", "-"): v for k, v in kw_args.items()}
